using System;
using System.Collections.Generic;
using System.Linq;
using System.IO;
using System.Web.Mvc;
using Syncfusion.Presentation;
using Syncfusion.XlsIO;
using System.Drawing;
using CultureAudit.Models;
using Syncfusion.XlsIO.Implementation.Charts;
using Syncfusion.OfficeChart;

namespace CultureAudit.Areas.Admin.Controllers
{
    public class PowerPointController : Controller
    {
        CultureAuditEntities db = new CultureAuditEntities();

        // GET: Admin/PowerPoint
        public ActionResult Index()
        {
            return View();
        }


        public PresentationResult DemographicsPresentation(int SurveyNumber, int Demographic)
        {
            SurveyDefinition survey = new SurveyDefinition();
            string companyName = survey.GetCompany(SurveyNumber);
            string CompanyNoSpaceName = companyName.Replace(" ", "_");
            string demoQuestion = survey.GetQuestionText(Demographic);
            string SurveyName = survey.GetSurveyName(SurveyNumber);

            // Because each demographic question may have a different number of possible responses we need to find out how many there are for this particular demographic
            int NumberPossibleResponses = survey.NumberDemographicResponses(Demographic);

            // Because each demographic is different we have to get a list of the possible responses
            GetDemographicResponses gdr = new GetDemographicResponses();
            gdr.Demographic = Demographic;
            List<GetDemographicResponses_Result> DemoResponse = gdr.DemographicResponses();

            //Set a base height for the bar charts
            int baseHeight = 57;

            // 
            int z = 1;
            // Cannot get data directly from database for charts.   It can only come from spreadsheets so we will create a spreadsheet which we will delete after the presentation has been created 
            using (ExcelEngine excelEngine = new ExcelEngine())
            {

                // Set the default application version as Excel 2016.
                excelEngine.Excel.DefaultVersion = ExcelVersion.Excel2016;

                IWorkbook workbook = excelEngine.Excel.Workbooks.Create(new string[] { "Total", "Q1", "Q4", "Q5", "Q6", "Q7", "Q8", "Q9", "Q11", "Q12", "Q15", "Q17", "Q18", "Category Rank Comparison", "Rate Strongly Agree", "Rate Strongly Disagree" });

                // Total
                NumberByDemographics nbd = new NumberByDemographics();
                nbd.Demographic = Demographic;
                List<NumberByDemographics_Result> DemoTotal = nbd.DemographicNumbers();
                IWorksheet sheet_0 = workbook.Worksheets[0];
                sheet_0.ImportData(DemoTotal, 1, 1, true);

                // Q1
                IWorksheet sheet0 = workbook.Worksheets[1];
                IList<ChartData> data0 = survey.PivotedDemographics(1, SurveyNumber, Demographic, NumberPossibleResponses);
                
                // Display headers for each of the columns 
                sheet0[1, 1].Text = "Questions";
                foreach (var x in DemoResponse)
                {
                    sheet0[1, (z + x.Sequence)].Text = x.ResponseText;
                }
                // Display data on worksheet
                sheet0.ImportData(data0, 2, 1, false);
              
                // Q4
                IWorksheet sheet1 = workbook.Worksheets[2];
                IList<ChartData> data1 = survey.PivotedDemographics(4, SurveyNumber, Demographic, NumberPossibleResponses);
                sheet1[1, 1].Text = "Questions";
                foreach (var x in DemoResponse)
                {
                    sheet1[1, (z + x.Sequence)].Text = x.ResponseText;
                }
                sheet1.ImportData(data1, 2, 1, false);

                // Q5
                IWorksheet sheet2 = workbook.Worksheets[3];
                IList<ChartData> data2 = survey.PivotedDemographics(5, SurveyNumber, Demographic, NumberPossibleResponses);
                sheet2[1, 1].Text = "Questions";
                foreach (var x in DemoResponse)
                {
                    sheet2[1, (z + x.Sequence)].Text = x.ResponseText;
                }
                sheet2.ImportData(data2, 2, 1, false);

                // Q6
                IWorksheet sheet3 = workbook.Worksheets[4];
                IList<ChartData> data3 = survey.PivotedDemographics(6, SurveyNumber, Demographic, NumberPossibleResponses);
                if(data3.Count > 0)
                {
                    sheet3.ImportData(data3, 2, 1, false);
                    sheet3[1, 1].Text = "Questions";
                    foreach (var x in DemoResponse)
                    {
                        sheet3[1, (z + x.Sequence)].Text = x.ResponseText;
                    }
                }

                // Q7
                IWorksheet sheet4 = workbook.Worksheets[5];
                IList<ChartData> data4 = survey.PivotedDemographics(7, SurveyNumber, Demographic, NumberPossibleResponses);
                sheet4[1, 1].Text = "Questions";
                foreach (var x in DemoResponse)
                {
                    sheet4[1, (z + x.Sequence)].Text = x.ResponseText;
                }
                sheet4.ImportData(data4, 2, 1, false);
                // Q8
                IWorksheet sheet5 = workbook.Worksheets[6];
                IList<ChartData> data5 = survey.PivotedDemographics(8, SurveyNumber, Demographic, NumberPossibleResponses);
                sheet5[1, 1].Text = "Questions";
                foreach (var x in DemoResponse)
                {
                    sheet5[1, (z + x.Sequence)].Text = x.ResponseText;
                }
                sheet5.ImportData(data5, 2, 1, false);

                // Q9
                IWorksheet sheet6 = workbook.Worksheets[7];
                IList<ChartData> data6 = survey.PivotedDemographics(9, SurveyNumber, Demographic, NumberPossibleResponses);
                sheet6.ImportData(data6, 2, 1, false);
                sheet6[1, 1].Text = "Questions";
                foreach (var x in DemoResponse)
                {
                    sheet6[1, (z + x.Sequence)].Text = x.ResponseText;
                }
                sheet6.ImportData(data6, 2, 1, false);

                // Q11
                IWorksheet sheet7 = workbook.Worksheets[8];
                IList<ChartData> data7 = survey.PivotedDemographics(11, SurveyNumber, Demographic, NumberPossibleResponses);
                sheet7.ImportData(data7, 2, 1, false);
                sheet7[1, 1].Text = "Questions";
                foreach (var x in DemoResponse)
                {
                    sheet7[1, (z + x.Sequence)].Text = x.ResponseText;
                }
                sheet7.ImportData(data7, 2, 1, false);
                // Q12
                IWorksheet sheet8 = workbook.Worksheets[9];
                IList<ChartData> data8 = survey.PivotedDemographics(12, SurveyNumber, Demographic, NumberPossibleResponses);
                sheet8[1, 1].Text = "Questions";
                foreach (var x in DemoResponse)
                {
                    sheet8[1, (z + x.Sequence)].Text = x.ResponseText;
                }
                sheet8.ImportData(data8, 2, 1, false);

                // Q15
                IWorksheet sheet9 = workbook.Worksheets[10];
                IList<ChartData> data9 = survey.PivotedDemographics(15, SurveyNumber, Demographic, NumberPossibleResponses);
                sheet9[1, 1].Text = "Questions";
                foreach (var x in DemoResponse)
                {
                    sheet9[1, (z + x.Sequence)].Text = x.ResponseText;
                }
                sheet9.ImportData(data9, 2, 1, false);

                // Q17
                IWorksheet sheet10 = workbook.Worksheets[11];
                IList<ChartData> data10 = survey.PivotedDemographics(17, SurveyNumber, Demographic, NumberPossibleResponses);
                sheet10[1, 1].Text = "Questions";
                foreach (var x in DemoResponse)
                {
                    sheet10[1, (z + x.Sequence)].Text = x.ResponseText;
                }
                sheet10.ImportData(data10, 2, 1, false);

                //Q18
                IWorksheet sheet11 = workbook.Worksheets[12];
                IList<ChartData> data11 = survey.PivotedDemographics(18, SurveyNumber, Demographic, NumberPossibleResponses);
                sheet11[1, 1].Text = "Questions";
                foreach (var x in DemoResponse)
                {
                    sheet11[1, (z + x.Sequence)].Text = x.ResponseText;
                }
                sheet11.ImportData(data11, 2, 1, false);


                //Category Rank comparison
                IWorksheet sheet12 = workbook.Worksheets[13];
                IList<ChartData> data12 = survey.PivotedAllGroupseByDemographic(SurveyNumber, Demographic, NumberPossibleResponses);
                sheet12[1, 1].Text = "Questions";
                foreach (var x in DemoResponse)
                {
                    sheet12[1, (z + x.Sequence)].Text = x.ResponseText;
                }
                sheet12.ImportData(data12, 2, 1, false);


                //Rate For Strongly Agree
                IWorksheet sheet13 = workbook.Worksheets[14];
                IList<ChartData> data13 = survey.PivotedStronglyPercentageByDemographic("5", SurveyNumber, Demographic, NumberPossibleResponses);
                sheet13[1, 1].Text = "Questions";
                foreach (var x in DemoResponse)
                {
                    sheet13[1, (z + x.Sequence)].Text = x.ResponseText;
                }
                sheet13.ImportData(data13, 2, 1, false);


                //Rate For Strongly Disagree
                IWorksheet sheet14 = workbook.Worksheets[15];
                IList<ChartData> data14 = survey.PivotedStronglyPercentageByDemographic("1", SurveyNumber, Demographic, NumberPossibleResponses);
                sheet14[1, 1].Text = "Questions";
                foreach (var x in DemoResponse)
                {
                    sheet14[1, (z + x.Sequence)].Text = x.ResponseText;
                }
                sheet14.ImportData(data14, 2, 1, false);



                //Save the spreadsheet on the server
                workbook.SaveAs(Server.MapPath("/App_Data/" + CompanyNoSpaceName + "Demographic.xlsx"));
            }
            //This is the excel spreadsheet we just created
            string file = Server.MapPath("/App_Data/" + CompanyNoSpaceName + "Demographic.xlsx");
            ExcelEngine excelEngine2 = new ExcelEngine();
            //Open the excel application.
            IApplication application = excelEngine2.Excel;

            IWorkbook destworkbook = application.Workbooks.Open(file);

            //Open a powerpoint presentation
            IPresentation presentation = Presentation.Create();

            //Add a slide
            ISlide TitleSlide = presentation.Slides.Add(SlideLayoutType.Blank);

            //Adds a textbox in a slide by specifying its position and size
            Syncfusion.Presentation.IShape textShape = TitleSlide.AddTextBox(100, 75, 756, 800);

            //Adds a paragraph into the textShape
            IParagraph paragraph = textShape.TextBody.AddParagraph();

            //Add some text to it.
            ITextPart textPart = paragraph.AddTextPart(companyName);

            //Set the horizontal alignment of paragraph
            paragraph.HorizontalAlignment = HorizontalAlignmentType.Center;

            //Applies font formatting to the text
            textPart.Font.FontSize = 80;
            textPart.Font.Bold = true;

            //Add survey name below company name 
            IParagraph titleParagraph = textShape.TextBody.AddParagraph();
            ITextPart Title = titleParagraph.AddTextPart("\r\n" + SurveyName);
            titleParagraph.HorizontalAlignment = HorizontalAlignmentType.Center;
            Title.Font.FontSize = 50;
            Title.Font.Bold = true;
            
            //Add question text below survey Name
            IParagraph qParagraph = textShape.TextBody.AddParagraph();
            ITextPart dq = qParagraph.AddTextPart("\r\n"+demoQuestion.Replace("#Company#", companyName));
            dq.Font.FontSize = 30;
            qParagraph.HorizontalAlignment = HorizontalAlignmentType.Center;


            // Add a chart
            Stream stream = new FileStream(file, FileMode.Open);
            stream.Position = 0;

            IWorksheet worksheet = destworkbook.Worksheets[0];
            worksheet.UsedRangeIncludesFormatting = false;

            //Get The range of values
            var totaldata = worksheet.UsedRange;

            //create new slide showing chart
            ISlide totalslide = presentation.Slides.Add(SlideLayoutType.Blank);
            IPresentationChart totalChart = totalslide.Charts.AddChart(stream, 1, totaldata.Address, new Rectangle(30, 15, 850, 500));
            totalChart.ChartType = Syncfusion.OfficeChart.OfficeChartType.Column_3D;
            totalChart.HasLegend = false;
            totalChart.ChartTitle = "Total Number For Each Group";
            totalChart.PrimaryValueAxis.Title = "Number Responsents";


            stream.Dispose();
            //Remove Total
            destworkbook.Worksheets[0].Remove();
            destworkbook.SaveAs(file);
            destworkbook.Close();
            destworkbook = application.Workbooks.Open(file);

            
            IRow newRow;
            ICell cell;
            int chartHeight;
            int numRows;

            GroupsListed GL = new GroupsListed();
            List<GroupList_Result> Groups = GL.GroupsList();
            foreach(var grp in Groups)
            {
                z = 1;
                if (Convert.ToInt16(grp.GroupNumber) == 6)
                {
                    if (!Convert.ToBoolean(grp.UsesMatrix))
                    {
                        destworkbook.Worksheets[0].Remove();
                        destworkbook.SaveAs(file);
                        destworkbook.Close();
                        destworkbook = application.Workbooks.Open(file);
                    }
                }
                if (Convert.ToBoolean(grp.UsesMatrix))
                {
                    int GroupNum = Convert.ToInt16(grp.GroupNumber);
                    //Create the slide 
                    ISlide Slide = presentation.Slides.Add(SlideLayoutType.Blank);
                    Syncfusion.Presentation.IShape shape = Slide.AddTextBox(50, 25, 856, 30);

                    dynamic GroupData = survey.GetPivotedDemographics(GroupNum, SurveyNumber, Demographic); // List of data  pulled from a SPROC with dynamic SQL
                    string GroupText = survey.GetGroupText(GroupNum);                                       // Header for this group of questions 
                    numRows = survey.CountMatrixQuestions(SurveyNumber, GroupNum);                          // Number of rows for table
 
                    IParagraph p = shape.TextBody.AddParagraph();
                    ITextPart txtPart = p.AddTextPart(GroupText);
                    p.HorizontalAlignment = HorizontalAlignmentType.Center;
                    txtPart.Font.Bold = true;

                    //Add a table to the slide
                    ITable tbl = Slide.Shapes.AddTable((numRows + 1), (NumberPossibleResponses + 1), 40, 70, 856, 500);

                    // Header row for the table -- The data is dynamic there will be different text based on the demographic 
                    newRow = tbl.Rows[0];
                    newRow.Height = 30;
                    cell = tbl[0, 0];
                    cell.TextBody.AddParagraph("");
                    cell.ColumnWidth = 300;
                    // Add the demographic text 
                    foreach (var dr in DemoResponse)
                    {
                        cell = tbl[0, dr.Sequence];
                        cell.TextBody.AddParagraph(dr.ResponseText);
                        cell.ColumnWidth = 100;
                        ITextPart text = cell.TextBody.Paragraphs[0].TextParts[0];
                        text.Font.FontSize = 10;
                    }
                    //Now we can add the actual numbers of responses for each question answered
                    foreach (var row in (dynamic)GroupData)
                    {
                        int y = 0;
                        foreach (var column in row)
                        {
                            //display the information in this row
                            newRow = tbl.Rows[z];
                            newRow.Height = 40;
                            cell = tbl[z, y];
                            cell.TextBody.AddParagraph(column.Value.ToString());
                            ITextPart text = cell.TextBody.Paragraphs[0].TextParts[0];
                            text.Font.FontSize = 10;
                            y++;  // column increment
                        }
                        z++; // row increment
                    }

                    // Add a chart
                    stream = new FileStream(file, FileMode.Open);
                    stream.Position = 0;

                    //Gets the worksheet
                    IWorksheet sheet = destworkbook.Worksheets[0];
                    sheet.UsedRangeIncludesFormatting = false;

                    //Get The range of values
                    var avaData = sheet.UsedRange;
                    if (numRows > 9)
                    {
                        baseHeight = 50;
                    }
                    if (numRows < 4)
                    {
                        baseHeight = 70;
                    }
                    chartHeight = numRows * baseHeight;

                    //create new slide showing chart
                    ISlide chartslide = presentation.Slides.Add(SlideLayoutType.Blank);
                    IPresentationChart chart = chartslide.Charts.AddChart(stream, 1, avaData.Address, new Rectangle(30, 15, 850, chartHeight));
                    chart.ChartType = Syncfusion.OfficeChart.OfficeChartType.Bar_Clustered;
                    chart.HasLegend = true;
                    chart.ChartTitle = GroupText;
                    chart.PrimaryValueAxis.Title = "Average Answer";
                    chart.PrimaryValueAxis.MinimumValue = 0;
                    chart.PrimaryValueAxis.MaximumValue = 5;
                
                    stream.Dispose();
                    //Remove worksheet  --  do this until Syncfusion fixes bug reading dynamic info from any other worksheet index
                    destworkbook.Worksheets[0].Remove();
                    destworkbook.SaveAs(file);
                    destworkbook.Close();
                    destworkbook = application.Workbooks.Open(file);
                 
                }
            }
         

            // ---------------------------------------------------------------------------------------------------------------------------- Category Rank Comparison
            //Create the new slide
            ISlide Slide13 = presentation.Slides.Add(SlideLayoutType.Blank);
            Syncfusion.Presentation.IShape textShape13 = Slide13.AddTextBox(50, 25, 856, 30);

            dynamic CRData = survey.GetAllGroupsByDemographic(SurveyNumber, Demographic);
            //Get the text to put in header
            string HeaderText = "Category Rank Comparison";
            numRows = 13;


            IParagraph paragraph13 = textShape13.TextBody.AddParagraph();
            //Add above text to this slide above the table
            ITextPart textPart13 = paragraph13.AddTextPart(HeaderText);
            paragraph13.HorizontalAlignment = HorizontalAlignmentType.Center;
            textPart13.Font.Bold = true;

            ITable table13 = Slide13.Shapes.AddTable((numRows + 1), (NumberPossibleResponses + 1), 40, 70, 856, 500);
            newRow = table13.Rows[0];
            newRow.Height = 30;
            cell = table13[0, 0];
            cell.TextBody.AddParagraph("");
            cell.ColumnWidth = 300;
            foreach (var dr in DemoResponse)
            {
                cell = table13[0, dr.Sequence];
                cell.TextBody.AddParagraph(dr.ResponseText);
                cell.ColumnWidth = 100;
                ITextPart text = cell.TextBody.Paragraphs[0].TextParts[0];
                text.Font.FontSize = 10;
            }
            z = 1;
            foreach (var row in (dynamic)CRData)
            {
                int y = 0;
                foreach (var column in row)
                {
                    //display the information in this row
                    newRow = table13.Rows[z];
                    newRow.Height = 30;
                    cell = table13[z, y];
                    cell.TextBody.AddParagraph(column.Value.ToString());
                    ITextPart text = cell.TextBody.Paragraphs[0].TextParts[0];
                    text.Font.FontSize = 10;
                    y++;  // column increment
                }
                z++; // row increment
            }

            // Add a chart
            stream = new FileStream(file, FileMode.Open);
            stream.Position = 0;

            //Gets the Category Rank Comparison worksheet
            IWorksheet worksheet13 = destworkbook.Worksheets[0];
            worksheet13.UsedRangeIncludesFormatting = false;

            //Get The range of values
            var data_13 = worksheet13.UsedRange;
            
            chartHeight = 500;

            //create new slide showing chart
            ISlide chart13slide = presentation.Slides.Add(SlideLayoutType.Blank);
            IPresentationChart chart13 = chart13slide.Charts.AddChart(stream, 1, data_13.Address, new Rectangle(30, 15, 850, chartHeight));
            chart13.ChartType = Syncfusion.OfficeChart.OfficeChartType.Column_Clustered_3D;
            chart13.HasLegend = true;
            chart13.ChartTitle = HeaderText;
            chart13.PrimaryValueAxis.Title = "Average Answer";
            chart13.PrimaryValueAxis.MinimumValue = 0;
            chart13.PrimaryValueAxis.MaximumValue = 5;


            stream.Dispose();
            //Remove Category Rank Comparison
            destworkbook.Worksheets[0].Remove();
            destworkbook.SaveAs(file);
            destworkbook.Close();
            destworkbook = application.Workbooks.Open(file);

            // Add a chart
            stream = new FileStream(file, FileMode.Open);
            stream.Position = 0;

            //------------------------------------------------------------------------------------------------------------------------------------Gets the Strongly Agree worksheet
            IWorksheet worksheet14 = destworkbook.Worksheets[0];
            worksheet14.UsedRangeIncludesFormatting = false;

            //Get The range of values
            var data_14 = worksheet14.UsedRange;
            HeaderText = "Rate For Strongly Agree";
            chartHeight = 500;

            //create new slide showing chart
            ISlide chart14slide = presentation.Slides.Add(SlideLayoutType.Blank);
            IPresentationChart chart14 = chart14slide.Charts.AddChart(stream, 1, data_14.Address, new Rectangle(30, 15, 850, chartHeight));
            chart14.ChartType = Syncfusion.OfficeChart.OfficeChartType.Column_Clustered_3D;
            chart14.HasLegend = true;
            chart14.ChartTitle = HeaderText;
            chart14.PrimaryValueAxis.Title = "Percentage  Strongly  Agree";
            chart14.PrimaryValueAxis.MinimumValue = 0;
            chart14.PrimaryValueAxis.MaximumValue = 100;

            
            stream.Dispose();
            //Remove Strongly Agree
            destworkbook.Worksheets[0].Remove();
            destworkbook.SaveAs(file);
            destworkbook.Close();
            destworkbook = application.Workbooks.Open(file);

            // Add a chart
            stream = new FileStream(file, FileMode.Open);
            stream.Position = 0;

            //---------------------------------------------------------------------------------------------------------------------------------Gets the Strongly Disagree worksheet
            IWorksheet worksheet15 = destworkbook.Worksheets[0];
            worksheet15.UsedRangeIncludesFormatting = false;

            //Get The range of values
            var data_15 = worksheet15.UsedRange;
            HeaderText = "Rate For Strongly Disagree";
            chartHeight = 500;

            //create new slide showing chart
            ISlide chart15slide = presentation.Slides.Add(SlideLayoutType.Blank);
            IPresentationChart chart15 = chart15slide.Charts.AddChart(stream, 1, data_15.Address, new Rectangle(30, 15, 850, chartHeight));
            chart15.ChartType = Syncfusion.OfficeChart.OfficeChartType.Column_Clustered_3D;
            chart15.HasLegend = true;
            chart15.ChartTitle = HeaderText;
            chart15.PrimaryValueAxis.Title = "Percentage  Strongly  Disagree";
            chart15.PrimaryValueAxis.MinimumValue = 0;
            chart15.PrimaryValueAxis.MaximumValue = 100;


            //foreach (IChartSerie serie in chart15.Series)
            //{
            //    int dataPointCount = (serie.EnteredDirectlyValues == null) ? serie.Values.Rows.Length : serie.EnteredDirectlyValues.Length;
            //    for (int dataPointPos = 0; dataPointPos < dataPointCount; dataPointPos++)
            //    {
            //        IChartDataPoint dataPoints = serie.DataPoints[dataPointPos];
            //        double value;
            //        if (serie.EnteredDirectlyValues == null)
            //            value = serie.Values.Rows[dataPointPos].Number;
            //        else
            //            value = (double)serie.EnteredDirectlyValues.GetValue(dataPointPos);

            //        (dataPoints.DataLabels as ChartDataLabelsImpl).Text = value.ToString() + "%";
            //    }
            //}

            stream.Dispose();
           
            //close the workbook
            destworkbook.Close();

            excelEngine2.Dispose();

            //Now delete the Excel file used to create the charts
            //if ((System.IO.File.Exists(file)))
            //{
            //    System.IO.File.Delete(file);
            //}


            return new PresentationResult(presentation, CompanyNoSpaceName+"Demographic.pptx", HttpContext.ApplicationInstance.Response);
        }

        public PresentationResult ReportPresentation(int SurveyNumber)
        {
            SurveyDefinition survey = new SurveyDefinition();
            string companyName = survey.GetCompany(SurveyNumber);
            string CompanyNoSpaceName = companyName.Replace(" ", "_");

            string SurveyName = survey.GetSurveyName(SurveyNumber);
            string NoSpaceName = SurveyName.Replace(" ", "_");
            // the following variables will be used to display the info in the table
            int sd;
            int d;
            int nand;
            int a;
            int sa;
            int sdpts;
            int dpts;
            int nandpts;
            int apts;
            int sapts;
            int total;
            int totalpts;
            string percentsd;
            string percentd;
            string percentnand;
            string percenta;
            string percentsa;
            double avg;
          

            // Cannot get data directly from database for charts.   It can only come from spreadsheets so we will create a spreadsheet which we will delete after the presentation has been created 
            using (ExcelEngine excelEngine = new ExcelEngine())
            {
  
                // Set the default application version as Excel 2016.
                excelEngine.Excel.DefaultVersion = ExcelVersion.Excel2016;

                // Create a workbook with all needed worksheets.
                IWorkbook workbook = excelEngine.Excel.Workbooks.Create(new string[] { "Q1", "Q2","Q3","Q4", "Q5", "Q6", "Q7", "Q8", "Q9","Q10", "Q11", "Q12","Q13","Q14", "Q15","Q16", "Q17", "Q18","Q19","Company Average","CA Desc"," CA Asc","Strongly Agree","Strongly Disagree" });

                // Access first worksheet from the workbook instance.
                IWorksheet sheet0 = workbook.Worksheets[0];

                // Create instance of class  -- will reuse on each worksheet
                AverageAnswerByGroup mod = new AverageAnswerByGroup();

                // Assign variables needed for this worksheet
                mod.GroupNumber = 1;
                mod.SurveyNumber = SurveyNumber;

                IList<AverageGroupAnswer_Result> group1 = mod.AverageGroupScores();
                sheet0.ImportData(group1, 1, 1, true);

                // Access 2nd worksheet from the workbook instance.
                IWorksheet sheet1 = workbook.Worksheets[1];
                PhraseInstance phrase = new PhraseInstance();
                phrase.GroupNumber = 2;
                phrase.SurveyNumber = SurveyNumber;
                IList<DisplayInstancesOfPhrase_Result> DoesWell = phrase.PhrasesUsed();
                sheet1.ImportData(DoesWell, 1, 1, true);

                // Access 3rd worksheet from the workbook instance.
                IWorksheet sheet2 = workbook.Worksheets[2];
                phrase.GroupNumber = 3;
                phrase.SurveyNumber = SurveyNumber;
                IList<DisplayInstancesOfPhrase_Result> DoBetter = phrase.PhrasesUsed();
                sheet2.ImportData(DoBetter, 1, 1, true);

                // Access 4th worksheet from the workbook instance.
                IWorksheet sheet3 = workbook.Worksheets[3];
                mod.GroupNumber = 4;
                mod.SurveyNumber = SurveyNumber;

                IList<AverageGroupAnswer_Result> group4 = mod.AverageGroupScores();
                sheet3.ImportData(group4, 1, 1, true);


                // Access 5th worksheet from the workbook instance.
                IWorksheet sheet4 = workbook.Worksheets[4];
                mod.GroupNumber = 5;
                mod.SurveyNumber = SurveyNumber;

                IList<AverageGroupAnswer_Result> group5 = mod.AverageGroupScores();
                sheet4.ImportData(group5, 1, 1, true);

                // Access 6th worksheet from the workbook instance.
                IWorksheet sheet5 = workbook.Worksheets[5];
                mod.GroupNumber = 6;
                mod.SurveyNumber = SurveyNumber;

                IList<AverageGroupAnswer_Result> group6 = mod.AverageGroupScores();
                if (group6.Count > 0)
                {
                    sheet5.ImportData(group6, 1, 1, true);
                }

                // Access 7th worksheet from the workbook instance.
                IWorksheet sheet6 = workbook.Worksheets[6];
                mod.GroupNumber = 7;
                mod.SurveyNumber = SurveyNumber;

                IList<AverageGroupAnswer_Result> group7 = mod.AverageGroupScores();
                sheet6.ImportData(group7, 1, 1, true);

                // Access 8th worksheet from the workbook instance.
                IWorksheet sheet7 = workbook.Worksheets[7];
                mod.GroupNumber = 8;
                mod.SurveyNumber = SurveyNumber;

                IList<AverageGroupAnswer_Result> group8 = mod.AverageGroupScores();
                sheet7.ImportData(group8, 1, 1, true);

                // Access 9th worksheet from the workbook instance.
                IWorksheet sheet8 = workbook.Worksheets[8];
                mod.GroupNumber = 9;
                mod.SurveyNumber = SurveyNumber;

                IList<AverageGroupAnswer_Result> group9 = mod.AverageGroupScores();
                sheet8.ImportData(group9, 1, 1, true);

                // Access 9th worksheet from the workbook instance.
                RadioButtonAnswers rba = new RadioButtonAnswers();
                rba.SurveyNumber = SurveyNumber;
                rba.GroupNumber = 10;
                IWorksheet sheet9 = workbook.Worksheets[9];
                IList<GetRadioAnswerOccurances_Result> IdeasRadio = rba.RadioAnswers();
                sheet9.ImportData(IdeasRadio, 1, 1, true);

                // Access 10th worksheet from the workbook instance.
                IWorksheet sheet10 = workbook.Worksheets[10];
                mod.GroupNumber = 11;
                mod.SurveyNumber = SurveyNumber;

                IList<AverageGroupAnswer_Result> group11 = mod.AverageGroupScores();
                sheet10.ImportData(group11, 1, 1, true);

                // Access 11th worksheet from the workbook instance.
                IWorksheet sheet11 = workbook.Worksheets[11];
                mod.GroupNumber = 12;
                mod.SurveyNumber = SurveyNumber;

                IList<AverageGroupAnswer_Result> group12 = mod.AverageGroupScores();
                sheet11.ImportData(group12, 1, 1, true);

                // Access 12th worksheet from the workbook instance.
                IWorksheet sheet12 = workbook.Worksheets[12];
                CheckBoxAnswers cba = new CheckBoxAnswers();
                cba.GroupNumber = 13;
                cba.SurveyNumber = SurveyNumber;
                IList<DisplayCheckBoxInstances_Result> HowCommunicate = cba.CheckAllAnswers();
                sheet12.ImportData(HowCommunicate, 1, 1, true);

                // Access 13th worksheet from the workbook instance.
                IWorksheet sheet13 = workbook.Worksheets[13];
                cba.GroupNumber = 14;
                cba.SurveyNumber = SurveyNumber;
                IList<DisplayCheckBoxInstances_Result> PreferCommunicate = cba.CheckAllAnswers();
                sheet13.ImportData(PreferCommunicate, 1, 1, true);

                // Access 14th worksheet from the workbook instance.
                IWorksheet sheet14 = workbook.Worksheets[14];
                mod.GroupNumber = 15;
                mod.SurveyNumber = SurveyNumber;

                IList<AverageGroupAnswer_Result> group15 = mod.AverageGroupScores();
                sheet14.ImportData(group15, 1, 1, true);

                // Access 15th worksheet from the workbook instance.
                IWorksheet sheet15 = workbook.Worksheets[15];
                phrase.GroupNumber = 16;
                phrase.SurveyNumber = SurveyNumber;
                IList<DisplayInstancesOfPhrase_Result> CoreValues = phrase.PhrasesUsed();
                sheet15.ImportData(CoreValues, 1, 1, true);

                // Access 16th worksheet from the workbook instance.
                IWorksheet sheet16 = workbook.Worksheets[16];
                mod.GroupNumber = 17;
                mod.SurveyNumber = SurveyNumber;

                IList<AverageGroupAnswer_Result> group17 = mod.AverageGroupScores();
                sheet16.ImportData(group17, 1, 1, true);

                // Access 17th worksheet from the workbook instance.
                IWorksheet sheet17 = workbook.Worksheets[17];
                mod.GroupNumber = 18;
                mod.SurveyNumber = SurveyNumber;

                IList<AverageGroupAnswer_Result> group18 = mod.AverageGroupScores();
                sheet17.ImportData(group8, 1, 1, true);

                //  18th is Q19  demographics section.   
                //Access the 19th worksheet
                AllGroupAverages allAvg = new AllGroupAverages();
                allAvg.SurveyNumber = SurveyNumber;
                allAvg.SortOn = 0;
                IList<AllGroupAverages_Result> AllAverage = allAvg.GroupAverages();
                IWorksheet sheet19 = workbook.Worksheets[19];
                sheet19.ImportData(AllAverage, 1, 1, true);


                //Access the 20th worksheet
                IWorksheet sheet20 = workbook.Worksheets[20];
                allAvg.SortOn = 2;
                List<AllGroupAverages_Result> allDesc = allAvg.GroupAverages();
                sheet20.ImportData(allDesc, 1, 1, true);

                //Access the 21st worksheet
                IWorksheet sheet21 = workbook.Worksheets[21];
                allAvg.SortOn = 1;
                List<AllGroupAverages_Result> AllAsc = allAvg.GroupAverages();
                sheet21.ImportData(AllAsc, 1, 1, true);

                //Access the 22nd worksheet
                IWorksheet sheet22 = workbook.Worksheets[22];
                AllParticipantsStronglyPercentages apsp = new AllParticipantsStronglyPercentages();
                apsp.SurveyNumber = SurveyNumber;
                apsp.AnswerNumber = "5"; //Strongly Agree
                IList<AllParticipantsStronglyPercentages_Result> AllStronglyAgree = apsp.StronglyPercentages();
                sheet22.ImportData(AllStronglyAgree, 1, 1, true);

                //Access the 23rd worksheet
                IWorksheet sheet23 = workbook.Worksheets[23];
                apsp.AnswerNumber = "1"; // Strongly Disagree
                IList<AllParticipantsStronglyPercentages_Result> AllStronglyDisagree = apsp.StronglyPercentages();
                sheet23.ImportData(AllStronglyDisagree, 1, 1, true);

        
                //Save the spreadsheet on the server
                workbook.SaveAs(Server.MapPath("/App_Data/"+ NoSpaceName +".xlsx"));
            }
            //This is the excel spreadsheet we just created
            string file = Server.MapPath("/App_Data/" + NoSpaceName + ".xlsx");
            ExcelEngine excelEngine2 = new ExcelEngine();
            //Open the excel application.
            IApplication application = excelEngine2.Excel;

            IWorkbook destworkbook = application.Workbooks.Open(file);


            int numCompleted = survey.CountParticipants(SurveyNumber, "Y");
            int Section1 = survey.CountParticipants(SurveyNumber, "1");
            int Participants = numCompleted + Section1;

            //Open a powerpoint presentation
            IPresentation presentation = Presentation.Create();

            //Add cover slide
            ISlide Slide0 = presentation.Slides.Add(SlideLayoutType.Blank);
            Syncfusion.Presentation.IShape textShape = Slide0.AddTextBox(100, 75, 756, 800);
            IParagraph p = textShape.TextBody.AddParagraph();

            //Add some text to it.
            ITextPart txtPart = p.AddTextPart(companyName);

            //Set the horizontal alignment of paragraph
            p.HorizontalAlignment = HorizontalAlignmentType.Center;

            //Apply font formatting to the above text
            txtPart.Font.FontSize = 80;
            txtPart.Font.Bold = true;

            IParagraph titleParagraph = textShape.TextBody.AddParagraph();
            ITextPart Title = titleParagraph.AddTextPart("\r\n"+SurveyName);
            titleParagraph.HorizontalAlignment = HorizontalAlignmentType.Center;
            Title.Font.FontSize = 50;
            Title.Font.Bold = true;

            //Set a base height for the bar charts
            int baseHeight = 57;


            ITable table;
            IRow row;
            ICell cell;
            int chartHeight;
            int numRows;
            int z;
            Stream stream;
            GroupsListed GL = new GroupsListed();
            List<GroupList_Result> Groups = GL.GroupsList();
            foreach (var grp in Groups)
            {
                z = 1;
                int GroupNum = Convert.ToInt16(grp.GroupNumber);
                if (!Convert.ToBoolean(grp.UsesMatrix))
                {
                   switch (GroupNum)
                   {
                        case 2:
                            stream = new FileStream(file, FileMode.Open);
                            stream.Position = 0;
                            //Get the text to put in header
                            string Group2Text = survey.GetGroupText(2);

                            //Gets the Q10 worksheet
                            IWorksheet worksheet2 = destworkbook.Worksheets[0];
                            worksheet2.UsedRangeIncludesFormatting = false;

                            //Get The range of values
                            var data2 = worksheet2.UsedRange;

                            //create new slide showing chart
                            ISlide chart2slide = presentation.Slides.Add(SlideLayoutType.Blank);
                            IPresentationChart chart2= chart2slide.Charts.AddChart(stream, 1, data2.Address, new Rectangle(30, 50, 800, 400));
                            chart2.ChartType = Syncfusion.OfficeChart.OfficeChartType.Pie;
                            chart2.HasLegend = true;
                            chart2.ChartTitle = Group2Text;
                            stream.Dispose();
                            destworkbook.Worksheets[0].Remove();
                            destworkbook.SaveAs(file);
                            destworkbook.Close();
                            destworkbook = application.Workbooks.Open(file);
                            break;

                        case 3:
                            stream = new FileStream(file, FileMode.Open);
                            stream.Position = 0;
                            //Get the text to put in header
                            string Group3Text = survey.GetGroupText(3);

                            //Gets the Q10 worksheet
                            IWorksheet worksheet3 = destworkbook.Worksheets[0];
                            worksheet3.UsedRangeIncludesFormatting = false;

                            //Get The range of values
                            var data3 = worksheet3.UsedRange;

                            //create new slide showing chart
                            ISlide chart3slide = presentation.Slides.Add(SlideLayoutType.Blank);
                            IPresentationChart chart3 = chart3slide.Charts.AddChart(stream, 1, data3.Address, new Rectangle(30, 50, 800, 400));
                            chart3.ChartType = Syncfusion.OfficeChart.OfficeChartType.Pie;
                            chart3.HasLegend = true;
                            chart3.ChartTitle = Group3Text;
                            stream.Dispose();
                            destworkbook.Worksheets[0].Remove();
                            destworkbook.SaveAs(file);
                            destworkbook.Close();
                            destworkbook = application.Workbooks.Open(file);
                            break;

                        case 10:
                            stream = new FileStream(file, FileMode.Open);
                            stream.Position = 0;
                            //Get the text to put in header
                            string Group10Text = survey.GetGroupText(10);

                            //Gets the Q10 worksheet
                            IWorksheet worksheet10 = destworkbook.Worksheets[0];
                            worksheet10.UsedRangeIncludesFormatting = false;

                            //Get The range of values
                            var data10 = worksheet10.UsedRange;

                            //create new slide showing chart
                            ISlide chart10slide = presentation.Slides.Add(SlideLayoutType.Blank);
                            IPresentationChart chart10 = chart10slide.Charts.AddChart(stream, 1, data10.Address, new Rectangle(30, 50, 800, 400));
                            chart10.ChartType = Syncfusion.OfficeChart.OfficeChartType.Pie;
                            chart10.HasLegend = true;
                            chart10.ChartTitle = Group10Text;
                            stream.Dispose();
                            destworkbook.Worksheets[0].Remove();
                            destworkbook.SaveAs(file);
                            destworkbook.Close();
                            destworkbook = application.Workbooks.Open(file);
                            break;
                        case 13:

                            stream = new FileStream(file, FileMode.Open);
                            stream.Position = 0;

                            //Get the text to put in header
                            string Group13Text = survey.GetGroupText(13);

                            //Gets the Q10 worksheet
                            IWorksheet worksheet13 = destworkbook.Worksheets[0];
                            worksheet13.UsedRangeIncludesFormatting = false;

                            //Get The range of values
                            var data13 = worksheet13.UsedRange;

                            //create new slide showing chart
                            ISlide chart13slide = presentation.Slides.Add(SlideLayoutType.Blank);
                            IPresentationChart chart13 = chart13slide.Charts.AddChart(stream, 1, data13.Address, new Rectangle(30, 50, 800, 400));
                            chart13.ChartType = Syncfusion.OfficeChart.OfficeChartType.Pie;
                            chart13.HasLegend = true;
                            chart13.ChartTitle = Group13Text;
                            stream.Dispose();
                            destworkbook.Worksheets[0].Remove();
                            destworkbook.SaveAs(file);
                            destworkbook.Close();
                            destworkbook = application.Workbooks.Open(file);
                            break;
                        case 14:
                            stream = new FileStream(file, FileMode.Open);
                            stream.Position = 0;

                            //Get the text to put in header
                            string Group14Text = survey.GetGroupText(14);

                            //Gets the Q10 worksheet
                            IWorksheet worksheet14 = destworkbook.Worksheets[0];
                            worksheet14.UsedRangeIncludesFormatting = false;

                            //Get The range of values
                            var data14 = worksheet14.UsedRange;

                            //create new slide showing chart
                            ISlide chart14slide = presentation.Slides.Add(SlideLayoutType.Blank);
                            IPresentationChart chart14 = chart14slide.Charts.AddChart(stream, 1, data14.Address, new Rectangle(30, 50, 800, 400));
                            chart14.ChartType = Syncfusion.OfficeChart.OfficeChartType.Pie;
                            chart14.HasLegend = true;
                            chart14.ChartTitle = Group14Text;
                            stream.Dispose();
                            destworkbook.Worksheets[0].Remove();
                            destworkbook.SaveAs(file);
                            destworkbook.Close();
                            destworkbook = application.Workbooks.Open(file);
                            break;
                        default:
                            destworkbook.Worksheets[0].Remove();
                            destworkbook.SaveAs(file);
                            destworkbook.Close();
                            destworkbook = application.Workbooks.Open(file);
                            break;
                    }

                }

                if (Convert.ToBoolean(grp.UsesMatrix))
                {
                    
                    //create new slide
                    ISlide Slide = presentation.Slides.Add(SlideLayoutType.Blank);
                    Syncfusion.Presentation.IShape textShape1 = Slide.AddTextBox(50, 25, 856, 30);
                    IParagraph paragraph = textShape1.TextBody.AddParagraph();
                    //Get the text to put in header
                    string GroupText = survey.GetGroupText(GroupNum);

                    //Add above text to this slide above the table
                    ITextPart textPart = paragraph.AddTextPart(GroupText);
                    paragraph.HorizontalAlignment = HorizontalAlignmentType.Center;
                    textPart.Font.Bold = true;
                    //get the data for the table
                    var GroupData = db.SurveyScoresByGroup(GroupNum, SurveyNumber).ToList();
                    numRows = GroupData.Count();
                    z = 1;

                    //Add a table of info
                    table = Slide.Shapes.AddTable((numRows + 1), 8, 40, 70, 856, 500);
                    row = table.Rows[0];
                    row.Height = 30;
                    cell = table[0, 0];
                    cell.TextBody.AddParagraph("");
                    cell.ColumnWidth = 300;
                    cell = table[0, 1];
                    cell.TextBody.AddParagraph("Strongly\r\nDisagree");
                    cell.ColumnWidth = 75;
                    ITextPart text = cell.TextBody.Paragraphs[0].TextParts[0];
                    text.Font.FontSize = 10;
                    cell = table[0, 2];
                    cell.TextBody.AddParagraph("Disagree");
                    cell.ColumnWidth = 75;
                    text = cell.TextBody.Paragraphs[0].TextParts[0];
                    text.Font.FontSize = 10;
                    cell = table[0, 3];
                    cell.TextBody.AddParagraph("Neither Agree\r\nNor Disagree");
                    cell.ColumnWidth = 75;
                    text = cell.TextBody.Paragraphs[0].TextParts[0];
                    text.Font.FontSize = 10;
                    cell = table[0, 4];
                    cell.TextBody.AddParagraph("Agree");
                    cell.ColumnWidth = 75;
                    text = cell.TextBody.Paragraphs[0].TextParts[0];
                    text.Font.FontSize = 10;
                    cell = table[0, 5];
                    cell.TextBody.AddParagraph("Strongly\r\nAgree");
                    cell.ColumnWidth = 75;
                    text = cell.TextBody.Paragraphs[0].TextParts[0];
                    text.Font.FontSize = 10;
                    cell = table[0, 6];
                    cell.TextBody.AddParagraph("Total\r\nParticipants");
                    cell.ColumnWidth = 75;
                    text = cell.TextBody.Paragraphs[0].TextParts[0];
                    text.Font.FontSize = 10;
                    cell = table[0, 7];
                    cell.TextBody.AddParagraph("Weighted\r\nAverage");
                    cell.ColumnWidth = 75;
                    text = cell.TextBody.Paragraphs[0].TextParts[0];
                    text.Font.FontSize = 10;
                    foreach (var x in GroupData)
                    {
                        int y = 0;
                        //determine values for each variable
                        sd = Convert.ToInt16(x.StronglyDisagree);
                        sdpts = sd * 1;
                        if (sd > 0)
                        {
                            percentsd = string.Format("{0:N2}", (Convert.ToDouble(sd) / Convert.ToDouble(Participants)) * 100);
                        }
                        else
                        {
                            percentsd = "0%";
                        }

                        d = Convert.ToInt16(x.Disagree);
                        dpts = d * 2;
                        if (d > 0)
                        {
                            percentd = string.Format("{0:N2}", (Convert.ToDouble(d) / Convert.ToDouble(Participants)) * 100);
                        }
                        else
                        {
                            percentd = "0%";
                        }

                        nand = Convert.ToInt16(x.NAND);
                        nandpts = nand * 3;

                        if (nand > 0)
                        {
                            percentnand = string.Format("{0:N2}", (Convert.ToDouble(nand) / Convert.ToDouble(Participants)) * 100);
                        }
                        else
                        {
                            percentnand = "0%";
                        }

                        a = Convert.ToInt16(x.Agree);
                        apts = a * 4;

                        if (a > 0)
                        {
                            percenta = string.Format("{0:N2}%", (Convert.ToDouble(a) / Convert.ToDouble(Participants)) * 100);
                        }
                        else
                        {
                            percenta = "0%";
                        }

                        sa = Convert.ToInt16(x.StronglyAgree);
                        sapts = sa * 5;
                        if (sa > 0)
                        {
                            percentsa = string.Format("{0:N2}%", (Convert.ToDouble(sa) / Convert.ToDouble(Participants)) * 100);
                        }
                        else
                        {
                            percentsa = "0%";
                        }
                        total = sd + d + nand + a + sa;
                        totalpts = sdpts + dpts + nandpts + apts + sapts;
                        avg = Convert.ToDouble(totalpts) / Convert.ToDouble(total);


                        //display the information in this row
                        row = table.Rows[z];
                        row.Height = 40;
                        cell = table[z, y];
                        cell.TextBody.AddParagraph(x.QuestionText.Replace("#Company#", companyName));
                        text = cell.TextBody.Paragraphs[0].TextParts[0];
                        text.Font.FontSize = 10;
                        y++;  // column increment
                        cell = table[z, y];
                        cell.TextBody.AddParagraph(percentsd.ToString() + "\r\n" + sd.ToString());
                        text = cell.TextBody.Paragraphs[0].TextParts[0];
                        text.Font.FontSize = 10;
                        y++;
                        cell = table[z, y];
                        cell.TextBody.AddParagraph(percentd.ToString() + "\r\n" + d.ToString());
                        text = cell.TextBody.Paragraphs[0].TextParts[0];
                        text.Font.FontSize = 10;
                        y++;
                        cell = table[z, y];
                        cell.TextBody.AddParagraph(percentnand.ToString() + "\r\n" + nand.ToString());
                        text = cell.TextBody.Paragraphs[0].TextParts[0];
                        text.Font.FontSize = 10;
                        y++;
                        cell = table[z, y];
                        cell.TextBody.AddParagraph(percenta.ToString() + "\r\n" + a.ToString());
                        text = cell.TextBody.Paragraphs[0].TextParts[0];
                        text.Font.FontSize = 10;
                        y++;
                        cell = table[z, y];
                        cell.TextBody.AddParagraph(percentsa.ToString() + "\r\n" + sa.ToString());
                        text = cell.TextBody.Paragraphs[0].TextParts[0];
                        text.Font.FontSize = 10;
                        y++;
                        cell = table[z, y];
                        cell.TextBody.AddParagraph(total.ToString());
                        text = cell.TextBody.Paragraphs[0].TextParts[0];
                        text.Font.FontSize = 10;
                        y++;
                        cell = table[z, y];
                        cell.TextBody.AddParagraph(avg.ToString("N2"));
                        text = cell.TextBody.Paragraphs[0].TextParts[0];
                        text.Font.FontSize = 10;
                        z++;  //row increment
                    }

                    // Add a chart
                    stream = new FileStream(file, FileMode.Open);
                    stream.Position = 0;

                    //Gets the worksheet
                    IWorksheet worksheet = destworkbook.Worksheets[0];
                    worksheet.UsedRangeIncludesFormatting = false;

                    //Get The range of values
                    var grpdata = worksheet.UsedRange;
                    if (numRows > 9)
                    {
                        baseHeight = 50;
                    }
                    if (numRows < 4)
                    {
                        baseHeight = 70;
                    }
                    chartHeight = numRows * baseHeight;

                    //create new slide showing chart
                    ISlide chartslide = presentation.Slides.Add(SlideLayoutType.Blank);
                    IPresentationChart chart = chartslide.Charts.AddChart(stream, 1, grpdata.Address, new Rectangle(30, 15, 850, chartHeight));
                    chart.ChartType = Syncfusion.OfficeChart.OfficeChartType.Bar_Clustered;
                    chart.HasLegend = false;
                    chart.ChartTitle = GroupText;
                    chart.PrimaryValueAxis.Title = "Average Answer";
                    chart.PrimaryValueAxis.MinimumValue = 0;
                    chart.PrimaryValueAxis.MaximumValue = 5;

                    //Dispose of the file stream  we will recreate in a bit 
                    stream.Dispose();

                    //Remove worksheet
                    destworkbook.Worksheets[0].Remove();

                    //Now save it 
                    destworkbook.SaveAs(file);

                    //close it 
                    destworkbook.Close();
                    //reopen 
                    destworkbook = application.Workbooks.Open(file);
                }
            }


            // Add a chart
            stream = new FileStream(file, FileMode.Open);
            stream.Position = 0;


            //Gets the worksheet
            IWorksheet worksheet18 = destworkbook.Worksheets[0];
            worksheet18.UsedRangeIncludesFormatting = false;

            //Get The range of values
            var allAvgData = worksheet18.UsedRange;
            //create new slide showing chart
            ISlide cwaSlide = presentation.Slides.Add(SlideLayoutType.Blank);
            IPresentationChart cwaChart = cwaSlide.Charts.AddChart(stream, 1, allAvgData.Address, new Rectangle(30, 15, 850, 500));
            cwaChart.ChartType = Syncfusion.OfficeChart.OfficeChartType.Column_Clustered_3D;
            cwaChart.HasLegend = false;
            cwaChart.ChartTitle = "Company Wide Average For Each Group";
            cwaChart.PrimaryValueAxis.Title = "Average Answer";
            cwaChart.PrimaryValueAxis.MinimumValue = 0;
            cwaChart.PrimaryValueAxis.MaximumValue = 5;


            stream.Dispose();
     
            //Remove Company Wide Averages
            destworkbook.Worksheets[0].Remove();
            destworkbook.SaveAs(file);
            destworkbook.Close();
            destworkbook = application.Workbooks.Open(file);

            // Add a chart
            stream = new FileStream(file, FileMode.Open);
            stream.Position = 0;


            //Gets the worksheet
            IWorksheet worksheet19 = destworkbook.Worksheets[0];
            worksheet19.UsedRangeIncludesFormatting = false;

            //Get The range of values
            var allDescAvgData = worksheet19.UsedRange;
            //create new slide showing chart
            ISlide descSlide = presentation.Slides.Add(SlideLayoutType.Blank);
            IPresentationChart DescChart = descSlide.Charts.AddChart(stream, 1, allDescAvgData.Address, new Rectangle(30, 15, 850, 500));
            DescChart.ChartType = Syncfusion.OfficeChart.OfficeChartType.Column_Clustered_3D;
            DescChart.HasLegend = false;
            DescChart.ChartTitle = "High to Low Average For Each Group";
            DescChart.PrimaryValueAxis.Title = "Average Answer";
            DescChart.PrimaryValueAxis.MinimumValue = 0;
            DescChart.PrimaryValueAxis.MaximumValue = 5;


            stream.Dispose();

            //Remove Desc Averages
            destworkbook.Worksheets[0].Remove();
            destworkbook.SaveAs(file);
            destworkbook.Close();
            destworkbook = application.Workbooks.Open(file);

            // Add a chart
            stream = new FileStream(file, FileMode.Open);
            stream.Position = 0;


            //Gets the worksheet
            IWorksheet worksheet20 = destworkbook.Worksheets[0];
            worksheet20.UsedRangeIncludesFormatting = false;

            //Get The range of values
            var allAscData = worksheet20.UsedRange;
            //create new slide showing chart
            ISlide ascSlide = presentation.Slides.Add(SlideLayoutType.Blank);
            IPresentationChart ascChart = ascSlide.Charts.AddChart(stream, 1, allAscData.Address, new Rectangle(30, 15, 850, 500));
            ascChart.ChartType = Syncfusion.OfficeChart.OfficeChartType.Column_Clustered_3D;
            ascChart.HasLegend = false;
            ascChart.ChartTitle = "Low to High Average For Each Group";
            ascChart.PrimaryValueAxis.Title = "Average Answer";
            ascChart.PrimaryValueAxis.MinimumValue = 0;
            ascChart.PrimaryValueAxis.MaximumValue = 5;


            stream.Dispose();

            //Remove Asc Averages
            destworkbook.Worksheets[0].Remove();
            destworkbook.SaveAs(file);
            destworkbook.Close();
            destworkbook = application.Workbooks.Open(file);

            // Add a chart
            stream = new FileStream(file, FileMode.Open);
            stream.Position = 0;

            //Gets the worksheet
            IWorksheet worksheet21 = destworkbook.Worksheets[0];
            worksheet21.UsedRangeIncludesFormatting = false;

            //Get The range of values
            var saData = worksheet21.UsedRange;
            //create new slide showing chart
            ISlide saSlide = presentation.Slides.Add(SlideLayoutType.Blank);
            IPresentationChart saChart = saSlide.Charts.AddChart(stream, 1, saData.Address, new Rectangle(30, 15, 850, 500));
            saChart.ChartType = Syncfusion.OfficeChart.OfficeChartType.Column_Clustered_3D;
            saChart.HasLegend = false;
            saChart.ChartTitle = "Rate For Strongly Agree";
            saChart.PrimaryValueAxis.Title = "Percentage Strongly Agree";
            saChart.PrimaryValueAxis.MinimumValue = 0;
            saChart.PrimaryValueAxis.MaximumValue = 100;
            saChart.Series[0].DataPoints.DefaultDataPoint.DataFormat.Fill.ForeColor = Color.Green;
 
            stream.Dispose();

            //Remove strongly agree
            destworkbook.Worksheets[0].Remove();
            destworkbook.SaveAs(file);
            destworkbook.Close();
            destworkbook = application.Workbooks.Open(file);

            // Add a chart
            stream = new FileStream(file, FileMode.Open);
            stream.Position = 0;

            //Gets the worksheet
            IWorksheet worksheet22 = destworkbook.Worksheets[0];
            worksheet22.UsedRangeIncludesFormatting = false;

            //Get The range of values
            var sdData = worksheet22.UsedRange;
            //create new slide showing chart
            ISlide sdSlide = presentation.Slides.Add(SlideLayoutType.Blank);
            IPresentationChart sdChart = sdSlide.Charts.AddChart(stream, 1, sdData.Address, new Rectangle(30, 15, 850, 500));
            sdChart.ChartType = Syncfusion.OfficeChart.OfficeChartType.Column_Clustered_3D;
            sdChart.HasLegend = false;
            sdChart.ChartTitle = "Rate For Strongly Disagree";
            sdChart.PrimaryValueAxis.Title = "Percentage  Strongly  Disagree";
            sdChart.PrimaryValueAxis.MinimumValue = 0;
            sdChart.PrimaryValueAxis.MaximumValue = 100;
            sdChart.Series[0].DataPoints.DefaultDataPoint.DataFormat.Fill.ForeColor = Color.Red;
 
            stream.Dispose();

            foreach (ChartSerieImpl serie in sdChart.Series)
            {

                int dataPointCount = (serie.EnteredDirectlyValues == null) ? serie.Values.Rows.Length : serie.EnteredDirectlyValues.Length;
                for (int dataPointPos = 0; dataPointPos < dataPointCount; dataPointPos++)
                {
                    IChartDataPoint dataPoints = serie.DataPoints[dataPointPos];
                    double value;
                    if (serie.EnteredDirectlyValues == null)
                        value = serie.Values.Rows[dataPointPos].Number;
                    else
                        value = (double)serie.EnteredDirectlyValues.GetValue(dataPointPos);

                    (dataPoints.DataLabels as ChartDataLabelsImpl).Text = value.ToString() + "%";
                }

            }

            //close the workbook
            destworkbook.Close();

            excelEngine2.Dispose();

            //Now delete the Excel file used to create the charts
            //if ((System.IO.File.Exists(file)))
            //{
            //    System.IO.File.Delete(file);
            //}

            return new PresentationResult(presentation, NoSpaceName+".pptx", HttpContext.ApplicationInstance.Response);
        }


        public static void FillPercentage(IPresentationChart chart)
        {

            foreach (IChartSerie serie in chart.Series)
            {
                int dataPointCount = (serie.EnteredDirectlyValues == null) ? serie.Values.Rows.Length : serie.EnteredDirectlyValues.Length;
                for (int dataPointPos = 0; dataPointPos < dataPointCount; dataPointPos++)
                {
                    IChartDataPoint dataPoints = serie.DataPoints[dataPointPos];
                    double value;
                    if (serie.EnteredDirectlyValues == null)
                        value = serie.Values.Rows[dataPointPos].Number;
                    else
                        value = (double)serie.EnteredDirectlyValues.GetValue(dataPointPos);

                    (dataPoints.DataLabels as ChartDataLabelsImpl).Text = value.ToString() + "%";
                }
            }
        }

        public static void FillAverage(IChart chart)
        {

            foreach (IChartSerie serie in chart.Series)
            {
                int dataPointCount = (serie.EnteredDirectlyValues == null) ? serie.Values.Rows.Length : serie.EnteredDirectlyValues.Length;
                for (int dataPointPos = 0; dataPointPos < dataPointCount; dataPointPos++)
                {
                    IChartDataPoint dataPoints = serie.DataPoints[dataPointPos];
                    double value;
                    if (serie.EnteredDirectlyValues == null)
                        value = serie.Values.Rows[dataPointPos].Number;
                    else
                        value = (double)serie.EnteredDirectlyValues.GetValue(dataPointPos);

                    (dataPoints.DataLabels as ChartDataLabelsImpl).Text = value.ToString();
                }
            }
        }
    }
}
 